/*********************************************************************************************************
* Software License Agreement (BSD License)                                                               *
* Author: Sebastien Decugis <sdecugis@nict.go.jp>							 *
*													 *
* Copyright (c) 2008, WIDE Project and NICT								 *
* All rights reserved.											 *
* 													 *
* Redistribution and use of this software in source and binary forms, with or without modification, are  *
* permitted provided that the following conditions are met:						 *
* 													 *
* * Redistributions of source code must retain the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer.										 *
*    													 *
* * Redistributions in binary form must reproduce the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer in the documentation and/or other						 *
*   materials provided with the distribution.								 *
* 													 *
* * Neither the name of the WIDE Project or NICT nor the 						 *
*   names of its contributors may be used to endorse or 						 *
*   promote products derived from this software without 						 *
*   specific prior written permission of WIDE Project and 						 *
*   NICT.												 *
* 													 *
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED *
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR *
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 	 *
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 	 *
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR *
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF   *
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.								 *
*********************************************************************************************************/

/* Management of the list of peers for this extension */

#include "sec_nosec.h"

/* The peer names are stored in a simple list ordered alphabetically; 
 * this list is not supposed to be very long. Otherwise it can be changed to a hash table. */

typedef struct _sns_peer_ {
	 struct _sns_peer_ 	*next;
	 char			*diamid;
	 int			 prio;
 } sns_peer_t;
 
static sns_peer_t sentinel;

/* Note: we don't need to protect the list since we're not yet multithreaded when we add to it */


/* Initialize the hash table where the peers are stored */
int sns_init(void)
{
	TRACE_ENTRY("");
	
	memset(&sentinel, 0, sizeof(sns_peer_t));
	
	return ENOTSUP;
}

/* Add a new peer entry. The diametedid string is copied. */
int sns_addpeer( char * diameterid, int priority )
{
	int ret = -1;
	sns_peer_t * prev, *new;
	
	TRACE_ENTRY("%p %d", diameterid, priority);
	
	/* Search the correct place in the list. Diameter ID are FQDN, case-insensitive (RFC4343). */
	for (prev = &sentinel; (prev->next != NULL) && ( (ret = strcasecmp ( prev->next->diamid, diameterid )) < 0); prev = prev->next);
	
	/* Case of duplicate */
	if (ret == 0) {
		TRACE_DEBUG(INFO, "This diameter id is already in the list");
		return EALREADY;
	}
	
	/* Create a new element */
	new = malloc(sizeof(sns_peer_t));
	if (new == NULL) {
		log_error("Memory allocation failed: %s\n", strerror(errno));
		TRACE_DEBUG(INFO, "malloc failed");
		return ENOMEM;
	}
	memset(new, 0, sizeof(sns_peer_t));
	new->diamid = strdup(diameterid);
	if (new->diamid == NULL) {
		log_error("Memory allocation failed: %s\n", strerror(errno));
		TRACE_DEBUG(INFO, "malloc failed");
		free(new);
		return ENOMEM;
	}
	
	new->prio = priority;
	
	/* link it */
	new->next = prev->next;
	prev->next = new;
	
	/* Element added */
	return 0;
}

/* Find a peer in the table. Return the priority if found, -1 otherwise */
int sns_findpeer( char * diameterid )
{
	int ret = -1;
	sns_peer_t * item;
	
	TRACE_ENTRY("%p", diameterid);
	
	/* Search the element in the list. Diameter ID are FQDN, case-insensitive (RFC4343). */
	for (item = sentinel.next; (item != NULL) && ( (ret = strcasecmp ( item->diamid, diameterid )) < 0); item = item->next);
	
	if (ret == 0) {
		/* we found the element */
		return item->prio;
	} else {
		/* The peer is not in the list, so return -1 to tell that we don't support it */
		return -1;
	}
}

/* Destroy the table and free all data */
int sns_fini(void)
{
	sns_peer_t * item;
	
	TRACE_ENTRY("");
	
	while (sentinel.next != NULL) {
		item = sentinel.next;
		sentinel.next = item->next;
		free(item->diamid);
		free(item);
	}
	
	return 0;
}
